/*
 * TimeStamperView.java
 */

package jdbernard.timestamper;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import javax.swing.JFileChooser;
import org.jdesktop.application.Action;
import org.jdesktop.application.Application;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;

/**
 * The application's main frame.
 */
public class TimeStamperView extends FrameView implements MouseMotionListener {

    public TimeStamperView(SingleFrameApplication app) {
        super(app);

        initComponents();

        ResourceMap resourceMap = getResourceMap();

        getFrame().setUndecorated(true);

        fileChooser = new JFileChooser();

        updateTimer = new Timer();
        updateTimer.scheduleAtFixedRate(new TimerTask() {

            @Override
            public void run() {
                Date currentTime = new Date();
                currentTimeLabel.setText(Timeline.shortFormat.format(currentTime));
                if (mostRecentTask != null) {
                    long seconds = currentTime.getTime() - mostRecentTask.getTime();
                    seconds /= 1000;
                    long minutes = seconds / 60;
                    seconds = seconds % 60;
                    long hours = minutes / 60;
                    minutes %= 60;
                    long days = hours / 24;
                    hours %= 24;
                    StringBuilder sb = new StringBuilder();

                    if (days > 0) sb.append(Long.toString(days) + "day ");
                    if (hours > 0) sb.append(Long.toString(hours) + "hr ");
                    if (minutes > 0) sb.append(Long.toString(minutes) + "min ");
                    sb.append(Long.toString(seconds) + "sec");

                    totalTimeNow.setText(sb.toString());
                } else totalTimeNow.setText("");
            }
        }, 0, 1000);

        thinTaskFont = taskTextField.getFont().deriveFont(Font.PLAIN);
        boldTaskFont = thinTaskFont.deriveFont(Font.BOLD);

        // refresh new timeline
        Timeline t = ((TimeStamperApp) getApplication()).getActiveTimeline();
        Timeline.TimelineMarker lastMarker = t.getLastMarker(new Date());

        if (lastMarker != null) {
            mostRecentTask = lastMarker.getTimestamp();
            startTimeLabel.setText(Timeline.shortFormat.format(lastMarker.getTimestamp()));

            taskTextField.setText(lastMarker.getMark());
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        currentTaskLabel = new javax.swing.JLabel();
        exitButton = new javax.swing.JButton();
        taskTextField = new javax.swing.JTextField();
        startTimeLabel = new javax.swing.JLabel();
        totalTimeNow = new javax.swing.JLabel();
        currentTimeLabel = new javax.swing.JLabel();
        notesButton = new javax.swing.JButton();
        optionsButton = new javax.swing.JButton();
        optionsMenu = new javax.swing.JPopupMenu();
        saveTimelineMenuItem = new javax.swing.JMenuItem();
        saveTimelineAsMenuItem = new javax.swing.JMenuItem();
        loadTimelineMenuItem = new javax.swing.JMenuItem();

        mainPanel.setBorder(javax.swing.BorderFactory.createMatteBorder(2, 2, 2, 2, new java.awt.Color(0, 0, 0)));
        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                mainPanelMousePressed(evt);
            }
        });
        mainPanel.addMouseMotionListener(this);

        currentTaskLabel.setFont(currentTaskLabel.getFont().deriveFont(currentTaskLabel.getFont().getStyle() | java.awt.Font.BOLD, currentTaskLabel.getFont().getSize()+2));
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(jdbernard.timestamper.TimeStamperApp.class).getContext().getResourceMap(TimeStamperView.class);
        currentTaskLabel.setText(resourceMap.getString("currentTaskLabel.text")); // NOI18N
        currentTaskLabel.setName("currentTaskLabel"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(jdbernard.timestamper.TimeStamperApp.class).getContext().getActionMap(TimeStamperView.class, this);
        exitButton.setAction(actionMap.get("quit")); // NOI18N
        exitButton.setBorder(null);
        exitButton.setContentAreaFilled(false);
        exitButton.setFocusPainted(false);
        exitButton.setHideActionText(true);
        exitButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        exitButton.setName("exitButton"); // NOI18N
        exitButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                exitButtonMouseEntered(evt);
            }
            public void mouseExited(java.awt.event.MouseEvent evt) {
                exitButtonMouseExited(evt);
            }
        });

        taskTextField.setFont(taskTextField.getFont().deriveFont(taskTextField.getFont().getStyle() | java.awt.Font.BOLD));
        taskTextField.setText(resourceMap.getString("taskTextField.text")); // NOI18N
        taskTextField.setName("taskTextField"); // NOI18N
        taskTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                taskTextFieldKeyReleased(evt);
            }
        });

        startTimeLabel.setFont(startTimeLabel.getFont().deriveFont(startTimeLabel.getFont().getStyle() | java.awt.Font.BOLD, startTimeLabel.getFont().getSize()+4));
        startTimeLabel.setForeground(resourceMap.getColor("startTimeLabel.foreground")); // NOI18N
        startTimeLabel.setText(resourceMap.getString("startTimeLabel.text")); // NOI18N
        startTimeLabel.setName("startTimeLabel"); // NOI18N

        totalTimeNow.setFont(totalTimeNow.getFont().deriveFont(totalTimeNow.getFont().getStyle() | java.awt.Font.BOLD, totalTimeNow.getFont().getSize()+3));
        totalTimeNow.setForeground(resourceMap.getColor("totalTimeNow.foreground")); // NOI18N
        totalTimeNow.setText(resourceMap.getString("totalTimeNow.text")); // NOI18N
        totalTimeNow.setName("totalTimeNow"); // NOI18N

        currentTimeLabel.setFont(currentTimeLabel.getFont().deriveFont(currentTimeLabel.getFont().getStyle() | java.awt.Font.BOLD, currentTimeLabel.getFont().getSize()+3));
        currentTimeLabel.setForeground(resourceMap.getColor("currentTimeLabel.foreground")); // NOI18N
        currentTimeLabel.setText(resourceMap.getString("currentTimeLabel.text")); // NOI18N
        currentTimeLabel.setName("currentTimeLabel"); // NOI18N

        notesButton.setAction(actionMap.get("editNotes")); // NOI18N
        notesButton.setHideActionText(true);
        notesButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        notesButton.setName("notesButton"); // NOI18N

        optionsButton.setIcon(resourceMap.getIcon("optionsButton.icon")); // NOI18N
        optionsButton.setContentAreaFilled(false);
        optionsButton.setFocusPainted(false);
        optionsButton.setHideActionText(true);
        optionsButton.setIconTextGap(0);
        optionsButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        optionsButton.setMaximumSize(new java.awt.Dimension(16, 16));
        optionsButton.setMinimumSize(new java.awt.Dimension(16, 16));
        optionsButton.setName("optionsButton"); // NOI18N
        optionsButton.setPreferredSize(new java.awt.Dimension(16, 16));
        optionsButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                optionsButtonMouseClicked(evt);
            }
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                optionsButtonMouseEntered(evt);
            }
            public void mouseExited(java.awt.event.MouseEvent evt) {
                optionsButtonMouseExited(evt);
            }
        });

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                        .addComponent(totalTimeNow)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 12, Short.MAX_VALUE)
                        .addComponent(currentTimeLabel))
                    .addComponent(taskTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 229, Short.MAX_VALUE)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addComponent(currentTaskLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(startTimeLabel)))
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 109, Short.MAX_VALUE)
                        .addComponent(optionsButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(0, 0, 0)
                        .addComponent(exitButton))
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(notesButton)
                        .addContainerGap())))
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(exitButton, javax.swing.GroupLayout.PREFERRED_SIZE, 16, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(optionsButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(notesButton)
                            .addGroup(mainPanelLayout.createSequentialGroup()
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                    .addComponent(currentTaskLabel)
                                    .addComponent(startTimeLabel))
                                .addGap(0, 0, 0)
                                .addComponent(taskTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGap(0, 0, 0)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(currentTimeLabel)
                            .addComponent(totalTimeNow))))
                .addContainerGap(24, Short.MAX_VALUE))
        );

        optionsMenu.setName("optionsMenu"); // NOI18N

        saveTimelineMenuItem.setAction(actionMap.get("saveTimeline")); // NOI18N
        saveTimelineMenuItem.setName("saveTimelineMenuItem"); // NOI18N
        optionsMenu.add(saveTimelineMenuItem);

        saveTimelineAsMenuItem.setAction(actionMap.get("saveTimelineAs")); // NOI18N
        saveTimelineAsMenuItem.setName("saveTimelineAsMenuItem"); // NOI18N
        optionsMenu.add(saveTimelineAsMenuItem);

        loadTimelineMenuItem.setAction(actionMap.get("loadTimeline")); // NOI18N
        loadTimelineMenuItem.setName("loadTimelineMenuItem"); // NOI18N
        optionsMenu.add(loadTimelineMenuItem);

        setComponent(mainPanel);
    }// </editor-fold>//GEN-END:initComponents

private void taskTextFieldKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_taskTextFieldKeyReleased
    if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
        Timeline t = ((TimeStamperApp) getApplication()).getActiveTimeline();
        Date d = new Date();
        t.addMarker(d, taskTextField.getText(), "No comments.");
        startTimeLabel.setText(Timeline.shortFormat.format(d));
        taskTextField.setFont(boldTaskFont);
        mostRecentTask = d;
    } else {
        taskTextField.setFont(thinTaskFont);
    }
}//GEN-LAST:event_taskTextFieldKeyReleased

private void exitButtonMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_exitButtonMouseEntered
    exitButton.setIcon(getResourceMap().getIcon("exitButton.hover.icon"));
}//GEN-LAST:event_exitButtonMouseEntered

private void exitButtonMouseExited(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_exitButtonMouseExited
    exitButton.setIcon(getResourceMap().getIcon("exitButton.icon"));
}//GEN-LAST:event_exitButtonMouseExited

private void optionsButtonMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_optionsButtonMouseEntered
    optionsButton.setIcon(getResourceMap().getIcon("optionsButton.hover.icon"));//GEN-LAST:event_optionsButtonMouseEntered
}

private void optionsButtonMouseExited(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_optionsButtonMouseExited
    optionsButton.setIcon(getResourceMap().getIcon("optionsButton.icon"));//GEN-LAST:event_optionsButtonMouseExited
}

private void mainPanelMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mainPanelMousePressed
    mousePressRelativeToFrame = evt.getPoint();
}//GEN-LAST:event_mainPanelMousePressed

private void optionsButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_optionsButtonMouseClicked
    optionsMenu.show(evt.getComponent(), evt.getX(), evt.getY());
}//GEN-LAST:event_optionsButtonMouseClicked

    @Action
    public void editNotes() {
        // TODO:
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel currentTaskLabel;
    private javax.swing.JLabel currentTimeLabel;
    private javax.swing.JButton exitButton;
    private javax.swing.JMenuItem loadTimelineMenuItem;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JButton notesButton;
    private javax.swing.JButton optionsButton;
    private javax.swing.JPopupMenu optionsMenu;
    private javax.swing.JMenuItem saveTimelineAsMenuItem;
    private javax.swing.JMenuItem saveTimelineMenuItem;
    private javax.swing.JLabel startTimeLabel;
    private javax.swing.JTextField taskTextField;
    private javax.swing.JLabel totalTimeNow;
    // End of variables declaration//GEN-END:variables

    private Point mousePressRelativeToFrame;
    private JFileChooser fileChooser;
    private Timer updateTimer;
    private Font boldTaskFont;
    private Font thinTaskFont;
    private Date mostRecentTask;

    public void mouseDragged(MouseEvent e) {
        Point currentMousePoint = e.getLocationOnScreen();

        // this is the point we will compute as the new upper left
        // coordinate of the frame. It needs to be translated some to account
        // for the fact that the user can press the mouse anywhere on the
        // main panel.
        Point finalPoint = (Point) currentMousePoint.clone();
        // find out where the new point should be, ignoreing screen bounds
        finalPoint.translate(-mousePressRelativeToFrame.x,
            -mousePressRelativeToFrame.y);

        // get the current frame bounds
        Rectangle frameBounds = getFrame().getBounds();

        // screen resolution
        Dimension resolution = Toolkit.getDefaultToolkit().getScreenSize();

        // check to see if the frame is near any of the screen boundaries
        // if it is, 'snap' the final point to that boundary.
        if (frameBounds.x < 20)
            finalPoint.translate(-finalPoint.x, 0);
        if (frameBounds.y < 20)
            finalPoint.translate(0, -finalPoint.y);
        if ((resolution.width - (frameBounds.x + frameBounds.width)) < 20)
            finalPoint.translate(resolution.width -
                (finalPoint.x + frameBounds.width), 0);
        if ((resolution.height - (frameBounds.y + frameBounds.height)) < 20)
            finalPoint.translate(0, resolution.height -
                    (finalPoint.y + frameBounds.height));


        // this point represents a backwards version of the initial calculation
        // to find the finalPoint. It says, based on the current final point,
        // assume I moved the frame to that point. Where, relative to that point
        // should the mouse be, based on how far it was relative to the point
        // when the user pressed it.
        Point whereMouseShouldBe = (Point) finalPoint.clone();
        whereMouseShouldBe.translate(mousePressRelativeToFrame.x,
                mousePressRelativeToFrame.y);

        // if the actual mouse location is different from the expected location,
        // then we know that the snapping behaviour has altered the frames new
        // placement. If this alteration is too large (30 px apart in this case)
        // then we know the user is trying to pull the frame out of its snapped
        // position. In this case, we want to ignore the snap calculations and
        // base the new frame location entirely on the current mouse location
        if (whereMouseShouldBe.distance(currentMousePoint) > 30) {
            finalPoint = (Point) currentMousePoint.clone();
            finalPoint.translate(-mousePressRelativeToFrame.x,
                    -mousePressRelativeToFrame.y);
        }

        // once we've figured out where to go, put us there :)
        getFrame().setLocation(finalPoint);
    }

    public void mouseMoved(MouseEvent e) {
    }

    @Action
    public void saveTimeline() {
        ((TimeStamperApp) getApplication()).saveTimeline();
    }

    @Action
    public void saveTimelineAs() {
        if (fileChooser.showSaveDialog(getFrame())!=JFileChooser.APPROVE_OPTION)
            return;

        ((TimeStamperApp) getApplication()).saveTimelineToFile(
                fileChooser.getSelectedFile().getAbsolutePath());
    }

    @Action
    public void loadTimeline() {
        if (fileChooser.showOpenDialog(getFrame())!=JFileChooser.APPROVE_OPTION)
            return;

        ((TimeStamperApp) getApplication()).loadTimeline(
                fileChooser.getSelectedFile().getAbsolutePath());

        // refresh new timeline
        Timeline t = ((TimeStamperApp) getApplication()).getActiveTimeline();
        Timeline.TimelineMarker lastMarker = t.getLastMarker(new Date());

        if (lastMarker != null) {
            mostRecentTask = lastMarker.getTimestamp();
            startTimeLabel.setText(Timeline.shortFormat.format(lastMarker.getTimestamp()));

            taskTextField.setText(lastMarker.getMark());
        }
    }
}
